-- ClutterCleaner v0.2
-- BETA VERSION
-- Release date: 04-02-2015
-- Author: RevelatedDestiny

-- START OF VARIABLE INITIALIZATION --

local addonName = "ClutterCleaner" -- Used for addon load check.
local f = CreateFrame("Frame") -- Used for addon load check.
local loggedIn = false -- Used for addon load check.
local monsterSayFilterIDs = {} -- Keeps track of the zones in which to filter out CHAT_MSG_MONSTER_SAY.
local blizzardSystemFilterStrings = {} -- Keeps track of the strings to filter out in CHAT_MSG_SYSTEM.
local offlinePerson = nil -- Used in blizzardSystemFilter.

-- END OF VARIABLE INITIALIZATION --

-- START OF ADDON INITIALIZATION --
-- This initializes the addon when the player logs in or when addons get loaded (whichever comes first).
     
local function initialize(self, event, loadedAddonName, ...)
	if(not loggedIn and ((event == "ADDON_LOADED" and loadedAddonName == addonName) or (event == "PLAYER_LOGIN"))) then
		loggedIn = true
		f:UnregisterEvent("ADDON_LOADED")
		f:UnregisterEvent("PLAYER_LOGIN")
		f:SetScript("OnEvent", nil)
		ChatFrame_AddMessageEventFilter("CHAT_MSG_MONSTER_SAY", monsterSayFilter)
		ChatFrame_AddMessageEventFilter("CHAT_MSG_SYSTEM", blizzardSystemFilter)
		fillMonsterSayFilterIDs()
		fillBlizzardSystemFilterStrings()
	end
end
     
f:RegisterEvent("ADDON_LOADED")
f:RegisterEvent("PLAYER_LOGIN")
f:SetScript("OnEvent", initialize)
	
-- END OF ADDON INITIALIZATION --

-- Adds zones where NPC messages have to be filtered to monsterSayFilterIDs.
function fillMonsterSayFilterIDs()
	monsterSayFilterIDs["Warspear"] = 1011
	monsterSayFilterIDs["Stormshield"] = 1009
	monsterSayFilterIDs["Frostwall"] = 976
	monsterSayFilterIDs["Lunarfall"] = 971
end

-- Removes NPC say messages from zones listed in monsterSayFilterIDs.
function monsterSayFilter()
	local currentID = GetCurrentMapAreaID()
	for _,id in pairs(monsterSayFilterIDs) do
		if id == currentID then
			return true
		end
	end
	return false
end

-- Adds Blizzard's system messages to be filtered to the blizzardSystemFilterStrings array.
--  * ERR_CHAT_PLAYER_NOT_FOUND_S: "No player named '%s' is currently playing."
--    If you aren't receiving this immediately after sending a whisper, it is unwanted and will be filtered.
--  * ERR_NOT_IN_RAID: "You are not in a raid group"
--    If you are receiving this while in a raid group, it is unwanted and will be filtered.
function fillBlizzardSystemFilterStrings()
	blizzardSystemFilterStrings[ERR_CHAT_PLAYER_NOT_FOUND_S] =
		function()
			-- This can only return true if the person is yourself, your target, your focus target or
			-- in your party or raid, but seeing as this message usually pops up unwanted when you are in
			-- a party or raid thanks to unwanted side-effects of third party addons, this is no issue.
			return UnitIsConnected(offlinePerson)
		end
	blizzardSystemFilterStrings[ERR_NOT_IN_RAID] = 
		function()
			return IsInRaid()
		end
end

-- Removes some of Blizzard's system messages, namely those that show up at unwanted times.
function blizzardSystemFilter(_, _, message)
	if message:find("No player named") then
		local startIndex = message:find("'")
		local endIndex = message:find("'", startIndex+1)
		offlinePerson = message:sub(startIndex+1, endIndex-1)
		-- Change the format of "No player named '%s' is currently playing." to
		-- its format as listed in GlobalStrings.lua.
		message = message:gsub("'[^']*'", "'%%s'")
	end
	return blizzardSystemFilterStrings[message]
end